### ContentPackage: IMS Content Packaging support for Zope
### Copyright (C) 2002 Scott Wilson (s.wilson@bangor.ac.uk)
###
### This library is free software; you can redistribute it and/or
### modify it under the terms of the GNU Lesser General Public
### License as published by the Free Software Foundation; either
### version 2.1 of the License, or (at your option) any later version.
###
### This library is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
### Lesser General Public License for more details.
### 
### You should have received a copy of the GNU Lesser General Public
### License along with this library; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###
###
### This module contains the CPImporter class
### that provides import support - in the form of
### zip archives - enabling upload of content packages
### Based upon ZipFolder by Jens Quade.
###

import os
import re
import string
import tempfile
import types

import Manifest

import StringIO
import zipfile


MINZIPFILESIZE=22


class CPImporter:
    """A mix-in class to add IMS Content Package (i.e. Zip archive) import support to folders"""


    def import_zipfile(self,file):
        
        # minimum/maximum size check
        file.seek(0,2)
        fsize=file.tell()
        file.seek(0,0)
        if fsize > self.max_zipfile_size():
            raise RuntimeError,'ZIP file is too big'

        if fsize < MINZIPFILESIZE:
            raise RuntimeError,'ZIP file is too small'

        zf=zipfile.ZipFile(file,'r')

        if not(self.check_zip(zf)):
            raise RuntimeError,'Archive does not contain an IMS Manifest'

        try:      
            for name in zf.namelist():
                self._add_file_from_zip(zf,name)
                    
        finally:
            zf.close()


    def _no_manifest(self,zipfile):
        pass


    def _add_file_from_zip(self,zipfile,name):

        basename=os.path.basename(name)
        pathname=os.path.dirname(name)
        if not self.check_filename(pathname,basename):
            return # skip ugly files
        
        sf=StringIO.StringIO(zipfile.read(name))
        self.add_file(pathname,basename,sf)
        sf.close()

    #
    #
    # To be overwritten:
     
    def max_zipfile_size(self):
        """return maximum size for zip files. Default 180MB"""
        return 180*1024*1024;
    

    def check_zip(self,zipfile):
        # a hook to check zipfile before import loop
        # maybe: Look for a special file in the zip...
        
        pass

    
    def check_filename(self,pathname,basename):
        # return 0 to skip file, raise an exception to annoy...
        return 1;


    def add_file(self,pathname,basename,file):
        #add file to whatever...
        #is called with path,basename and file handle
        # overwrite this!
        pass
    
      

   
